/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2013 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

//***********************************************************************************************************
//  Nuvoton Technology Corp. 
//  E-Mail: MicroC-8bit@nuvoton.com
//  Date: 20/May/2013
//***********************************************************************************************************

/*===========================================================================================================
 * This file set up UART in mode 1. The baud rate is configured as 9600bps when the system clock frequency
 * is 22.1184MHz. The baud rate clock source can be Timer 1 or Timer 2 by configuration wizard setting.
 * This demo needs PC hyper-terminal and it should be configured as below.
 *----------------------
 * Bit per second: 9600
 * Data bits:      8
 * Parity:         None
 * Stop bits:      1
 * Flow control:   None
 * ---------------------
 *=========================================================================================================*/

/*
//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
//
// <o0.0> UART Baud Rate Source Select
//     <0=> Timer 1
//     <1=> Timer 2
//
//-------- <<< end of configuration section >>> ------------------------------
*/

#include <stdio.h>                  //for "printf"
#include "N78E366A.h"
#include "Typedef.h"

#define BAUD_SOURCE     0
#define BAUD_RATE       9600
#define U8BAUD_TIMER1   256-(22118400*2/BAUD_RATE/12/32)     //256-(Fsys*(2^SMOD)/Baud_Rate/32)
#define U16BAUD_TIMER2  65536-(22118400/BAUD_RATE/32)        //65536-(Fsys/Baud_Rate/32)

volatile BIT bTx_Busy = 0;

/************************************************************************************************************
*    UART interrupt subroutine
************************************************************************************************************/
void UART_ISR (void) interrupt 4    //interrupt address is 0x0023
{
    if (RI)                         //check Tx or Rx interrupt
    {
        RI = 0;                     //clear RI by software for next reception
        P0 = SBUF;                  //show received data, ASCII code of key
        P14 = ~P14;                 //P1.4 toggles for each received byte
    }

    else
    {
        TI = 0;                     //clear TI by software for next transmission
        bTx_Busy = 0;               //transmission complete, Tx not busy
        P15 = ~P15;                 //P1.5 toggles for each transmitted byte
    }
}

/************************************************************************************************************
*    Timer 2 interrupt subroutine
************************************************************************************************************/
void Timer2_ISR (void) interrupt 5  //interrupt address is 0x002B
{
    EXF2 = 0;
    P16 = ~P16;                     //P1.6 toggles for each T2EX pin falling edge 
}

#if BAUD_SOURCE
/************************************************************************************************************
*    Initial Timer 2
************************************************************************************************************/
void Timer2_Init (void)
{
    TCLK = 1;                       //select Timer 2 as transmission baud rate source
    RCLK = 1;                       //select Timer 2 as receive baud rate source
    TL2 = RCAP2L = U16BAUD_TIMER2;  //baud rate 9600bps@22.1184MHz
    TH2 = RCAP2H = U16BAUD_TIMER2 >> 8;

    EXEN2 = 1;                      //enable T2EX pin
    ET2 = 1;                        //Timer 2 interrupt enable, in baud rate generator mode, 
                                    //T2EX pin interrupt still works
    TR2 = 1;                        //Timer 2 run
}

#else                                                                                                                                   
/************************************************************************************************************
*    Initial Timer 1
************************************************************************************************************/
void Timer1_Init (void)
{
    TMOD = 0x20;                    //configure Timer 1 as auto-reload 8-bit mode
    PCON |= 0x80;                   //double baud rate enable
    TL1 = TH1 = U8BAUD_TIMER1;      //baud rate 9600bps@22.1184MHz
    TR1 = 1;                        //Timer 1 run
}

#endif

/************************************************************************************************************
*    Main function 
************************************************************************************************************/
void main (void)
{
    UINT8 u8Send_Data;

    P0OR |= 0x01;                   //enable P0 internal pull-up 

#if BAUD_SOURCE
    Timer2_Init();
#else
    Timer1_Init();
#endif

    SCON = 0x52;                    /*initial UART as mode 1, receive enable,
                                      TI should be set before using "printf"*/
    
    printf ("\n*========================================================*");
    printf ("\n*               N78E366A UART Mode 1 Demo                *");
    printf ("\n*========================================================*");
    printf ("\n 1. Press any key, the ASCII code will be shown on P0.");
    printf ("\n 2. Toggle P3.2 will display \"ABCD...\" sequentially.\n\n");

    ES = 1;                         //enable UART interrupt
    EA = 1;                         //enable global interrupt

    while(1)
    {
        /*send A~Z to PC, ASCII(A)=0x41, ASCII(Z)=0x5A*/
        for (u8Send_Data = 0x41; u8Send_Data <= 0x5A; u8Send_Data++)
        {
            while(P32);             //toggle P3.2
            while(!P32);

            while(bTx_Busy);        //check Tx busy?
            SBUF = u8Send_Data;     //send A~Z sequentially
            bTx_Busy = 1;           //during transmission, set Tx_busy 
        }
    }
}

